@@ -74,8 +74,8 @@ storeAnonymous reports = sequence_
       [ (report, repo, remoteRepo)
       | (report, repo@Repo { repoKind = Left remoteRepo }) <- rs ]
 
-storeLocal :: [PathTemplate] -> [(BuildReport, Repo)] -> IO ()
-storeLocal templates reports = sequence_
+storeLocal :: [PathTemplate] -> [(BuildReport, Repo)] -> Platform -> IO ()
+storeLocal templates reports platform = sequence_
   [ do createDirectoryIfMissing True (takeDirectory file)
        appendFile file output
        --TODO: make this concurrency safe, either lock the report file or make
@@ -94,6 +94,7 @@ storeLocal templates reports = sequence_
       where env = initialPathTemplateEnv
                     (BuildReport.package  report)
                     (BuildReport.compiler report)
+                    platform
 
     groupByFileName = map (\grp@((filename,_):_) -> (filename, map snd grp))
                     . groupBy (equating  fst)
@@ -116,12 +117,13 @@ fromPlanPackage :: Platform -> CompilerId
                 -> Maybe (BuildReport, Repo)
 fromPlanPackage (Platform arch os) comp planPackage = case planPackage of
 
-  InstallPlan.Installed pkg@(ConfiguredPackage (AvailablePackage {
-                          packageSource = RepoTarballPackage repo _ _ }) _ _) result
-    -> Just $ (BuildReport.new os arch comp pkg (Right result), repo)
+  InstallPlan.Installed pkg@(ReadyPackage (SourcePackage {
+                          packageSource = RepoTarballPackage repo _ _ }) _ _ _) result
+    -> Just $ (BuildReport.new os arch comp
+               (readyPackageToConfiguredPackage pkg) (Right result), repo)
 
-  InstallPlan.Failed pkg@(ConfiguredPackage (AvailablePackage {
-                       packageSource = RepoTarballPackage repo _ _ }) _ _) result
+  InstallPlan.Failed pkg@(ConfiguredPackage (SourcePackage {
+                       packageSource = RepoTarballPackage repo _ _ }) _ _ _) result
     -> Just $ (BuildReport.new os arch comp pkg (Left result), repo)
 
   _ -> Nothing
